#version 450
 #extension GL_EXT_multiview : enable
// Vk3DVision SinglePass
///////////////////////////////////////////////////////////////////////////////

#define STEREO 1

#define VIEW gl_ViewIndex

// Stereo and Mono Texture Sampling
#if(STEREO == 1)
	#define SAMPLER_2D sampler2DArray
	#define USAMPLER_2D usampler2DArray
	#define ISAMPLER_2D isampler2DArray
	#define IMAGE_2D image2DArray
	#define UIMAGE_2D uimage2DArray
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, vec3(uv, VIEW))
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, vec3(uv, VIEW), bias)
	#define IMAGE_LOAD(Image2D, pos) imageLoad(Image2D, ivec3(pos, VIEW))
	#define IMAGE_STORE(Image2D, pos, result) imageStore(Image2D, ivec3(pos, VIEW), result)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, ivec3(uv, VIEW), lod)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, vec3(uv, VIEW), lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, vec3(uv, VIEW), lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, vec3(uv, VIEW), comp)
	#define TEXTURE_GATHER1(Sampler, uv) textureGather(Sampler, vec3(uv, VIEW))
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, vec3(uv, VIEW), offset, comp)
	#define TEXTURE_GATHER_OFFSETS(Sampler, uv, offset) textureGatherOffsets(Sampler, vec3(uv, VIEW), offset)
	#define TEXTURE_GATHER_OFFSETS1(Sampler, uv, offset, comp) textureGatherOffsets(Sampler, vec3(uv, VIEW), offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod).xy
	#define TEXTURE_GRAD(Sampler, P, dPdx, dPdy) textureGrad(Sampler, vec3(P, VIEW), dPdx, dPdy)
#else
	#define SAMPLER_2D sampler2D
	#define USAMPLER_2D usampler2D
	#define ISAMPLER_2D isampler2D
	#define IMAGE_2D image2D
	#define UIMAGE_2D uimage2D
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, uv)
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, uv, bias)
	#define IMAGE_LOAD(Image2D, pos) imageLoad(Image2D, pos)
	#define IMAGE_STORE(Image2D, pos, result) imageStore(Image2D, pos, result)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, uv, lod)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, uv, lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, uv, lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, uv, comp)
	#define TEXTURE_GATHER1(Sampler, uv, comp) textureGather(Sampler, uv, comp)
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, uv, offset, comp)
	#define TEXTURE_GATHER_OFFSETS(Sampler, uv, offset) textureGatherOffsets(Sampler, uv, offset)
	#define TEXTURE_GATHER_OFFSETS1(Sampler, uv, offset, comp) textureGatherOffsets(Sampler, uv, offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod)
	#define TEXTURE_GRAD(Sampler, P, dPdx, dPdy) textureGrad(Sampler, P, dPdx, dPdy)
#endif 
 ///////////////////////////////////////////////////////////////////////////////


// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
struct Vk3DStereo
{
	vec4 stereo;
	vec4 custom_params;
};
layout(set = 0, binding = 30, std140) uniform Vk3DParams
{
	Vk3DStereo vk3d_params[2];
};

layout(set = 0, binding = 0, std140) uniform freqHigh_fragmentUniforms_ubo
{
    vec4 texsize;
    vec4 prevviewprojectionmatrixw;
    vec4 prevviewprojectionmatrixx;
    vec4 prevviewprojectionmatrixy;
    vec4 renderpositiontoviewtexture;
    vec4 downsampletype;
    vec4 filterweights0;
    vec4 filterweights1;
    vec4 filterweights3;
    vec4 filterweights4;
    vec4 filterweights2;
    vec4 filterweights5;
    vec4 prevtexsize;
    vec4 motionblurparms;
} freqHigh_fragmentUniforms;

layout(set = 0, binding = 1, std140) uniform freqLow_fragmentUniforms_ubo
{
    vec4 projectionmatrixz;
    vec4 frustumvectl;
    vec4 frustumvectr;
    vec4 resolutionscale;
    vec4 frustumvecbl;
    vec4 frustumvecbr;
    vec4 globalvieworigin;
} freqLow_fragmentUniforms;

layout(set = 0, binding = 2) uniform SAMPLER_2D samp_tex4;
layout(set = 0, binding = 3) uniform SAMPLER_2D samp_tex2;
layout(set = 0, binding = 4) uniform SAMPLER_2D samp_tex0;
layout(set = 0, binding = 5) uniform SAMPLER_2D samp_tex3;
layout(set = 0, binding = 6) uniform SAMPLER_2D samp_tex1;

layout(location = 0) out vec4 out_FragColor0;
layout(location = 1) out vec4 out_FragColor1;

vec2 screenPosToTexcoord(vec2 pos, vec4 bias_scale)
{
    return (pos * bias_scale.zw) + bias_scale.xy;
}

vec4 tex2DFetch(SAMPLER_2D image, ivec2 texcoord, int mip)
{
    return TEXEL_FETCH(image, texcoord, mip);
}

float GetLuma(vec3 c)
{
    return dot(c, vec3(0.2125999927520751953125, 0.715200006961822509765625, 0.072200000286102294921875));
}

vec3 saturate(vec3 v)
{
    return clamp(v, vec3(0.0), vec3(1.0));
}

vec3 toneMap(vec3 linearHDR)
{
    vec3 param = linearHDR;
    vec3 param_1 = linearHDR / vec3(128.0 + (1.0 * GetLuma(param)));
    vec3 toneMapped = saturate(param_1);
    return sqrt(toneMapped);
}

float saturate(float v)
{
    return clamp(v, 0.0, 1.0);
}

float fmax3(float f1, float f2, float f3)
{
    return max(f1, max(f2, f3));
}

float fmin3(float f1, float f2, float f3)
{
    return min(f1, min(f2, f3));
}

float GetLinearDepth(float ndcZ, vec4 projectionMatrixZ, float rcpFarZ, bool bFirstPersonArmsRescale)
{
    float linearZ = projectionMatrixZ.w / (ndcZ + projectionMatrixZ.z);
    if (bFirstPersonArmsRescale)
    {
        float _195;
        if (linearZ < 1.0)
        {
            _195 = 10.0;
        }
        else
        {
            _195 = 1.0;
        }
        linearZ *= _195;
    }
    return linearZ * rcpFarZ;
}

vec4 tex2Dlod(SAMPLER_2D image, vec4 texcoord)
{
    return TEXTURE_LOD(image, texcoord.xy, texcoord.w);
}

vec3 computeVelocity(inout vec2 winPos)
{
    const ivec2 offsets[4] = {ivec2(0,0), ivec2(0,0), ivec2(0,0), ivec2(0,0)};
    vec4 depths = TEXTURE_GATHER_OFFSETS(samp_tex4, winPos, offsets);
    vec3 offsetMinZ = vec3(vec2(-1.0), depths.x);
    vec3 _265;
    if (depths.y < offsetMinZ.z)
    {
        _265 = vec3(vec2(1.0, -1.0), depths.y);
    }
    else
    {
        _265 = offsetMinZ;
    }
    offsetMinZ = _265;
    vec3 _283;
    if (depths.z < offsetMinZ.z)
    {
        _283 = vec3(vec2(-1.0, 1.0), depths.z);
    }
    else
    {
        _283 = offsetMinZ;
    }
    offsetMinZ = _283;
    vec3 _300;
    if (depths.w < offsetMinZ.z)
    {
        _300 = vec3(vec2(1.0), depths.w);
    }
    else
    {
        _300 = offsetMinZ;
    }
    offsetMinZ = _300;
    winPos += (offsetMinZ.xy * freqHigh_fragmentUniforms.texsize.zw);
    float param = offsetMinZ.z;
    vec4 param_1 = freqLow_fragmentUniforms.projectionmatrixz;
    float param_2 = 1.0;
    bool param_3 = true;
    float linearZ = GetLinearDepth(param, param_1, param_2, param_3);
    vec3 frustumVecX0 = mix(freqLow_fragmentUniforms.frustumvectl.xyz, freqLow_fragmentUniforms.frustumvectr.xyz, vec3(winPos.x * freqLow_fragmentUniforms.resolutionscale.z));
    vec3 frustumVecX1 = mix(freqLow_fragmentUniforms.frustumvecbl.xyz, freqLow_fragmentUniforms.frustumvecbr.xyz, vec3(winPos.x * freqLow_fragmentUniforms.resolutionscale.z));
    vec3 frustumVec = mix(frustumVecX1, frustumVecX0, vec3(1.0 - (winPos.y * freqLow_fragmentUniforms.resolutionscale.w)));
    vec3 world_pos = freqLow_fragmentUniforms.globalvieworigin.xyz + (frustumVec * linearZ);
    float rcpHW = 1.0 / ((world_pos.x * freqHigh_fragmentUniforms.prevviewprojectionmatrixw.x) + ((world_pos.y * freqHigh_fragmentUniforms.prevviewprojectionmatrixw.y) + ((world_pos.z * freqHigh_fragmentUniforms.prevviewprojectionmatrixw.z) + freqHigh_fragmentUniforms.prevviewprojectionmatrixw.w)));
    vec2 winPosPrev = vec2((world_pos.x * freqHigh_fragmentUniforms.prevviewprojectionmatrixx.x) + ((world_pos.y * freqHigh_fragmentUniforms.prevviewprojectionmatrixx.y) + ((world_pos.z * freqHigh_fragmentUniforms.prevviewprojectionmatrixx.z) + freqHigh_fragmentUniforms.prevviewprojectionmatrixx.w)), (world_pos.x * freqHigh_fragmentUniforms.prevviewprojectionmatrixy.x) + ((world_pos.y * freqHigh_fragmentUniforms.prevviewprojectionmatrixy.y) + ((world_pos.z * freqHigh_fragmentUniforms.prevviewprojectionmatrixy.z) + freqHigh_fragmentUniforms.prevviewprojectionmatrixy.w))) * rcpHW;
    vec4 param_4 = vec4(winPos, 0.0, 0.0);
    vec2 objV = tex2Dlod(samp_tex2, param_4).xy;
    vec2 _471;
    if (objV.x < 1.0)
    {
        _471 = objV;
    }
    else
    {
        _471 = winPos - (winPosPrev * freqLow_fragmentUniforms.resolutionscale.xy);
    }
    return vec3(_471, linearZ);
}

vec3 toneMapInverse(vec3 toneMapped)
{
    vec3 linearHDR = toneMapped * toneMapped;
    vec3 param = linearHDR;
    return (linearHDR * 128.0) / vec3(1.0 - (1.0 * GetLuma(param)));
}

vec4 OutputVelocityTarget(vec4 mbParms, inout vec2 v, float z)
{
    vec2 v_orig = v;
    float alpha = 1.0;
    v *= (mbParms.z * alpha);
    float len = inversesqrt(dot(v, v));
    float param = mbParms.w * len;
    v *= saturate(param);
    v *= mbParms.xy;
    return vec4(v_orig, vec2(length(v), z));
}

void main()
{
    vec2 param = gl_FragCoord.xy;
    vec4 param_1 = freqHigh_fragmentUniforms.renderpositiontoviewtexture;
    vec2 tc = screenPosToTexcoord(param, param_1);
    vec3 curr_frame = vec3(0.0);
    vec3 curr_frame_lowpass = vec3(0.0);
    vec3 cMax = vec3(0.0);
    vec3 cMin = vec3(0.0);
    ivec2 param_2 = ivec2(gl_FragCoord.xy);
    int param_3 = 0;
    vec3 param_4 = tex2DFetch(samp_tex0, param_2, param_3).xyz;
    vec3 M = toneMap(param_4);
    ivec2 param_5 = ivec2(gl_FragCoord.xy);
    int param_6 = 0;
    vec3 param_7 = tex2DFetch(samp_tex3, param_5, param_6).xyz;
    float param_8 = length(M - toneMap(param_7)) * freqHigh_fragmentUniforms.downsampletype.z;
    float transparency_k = saturate(param_8);
    ivec2 param_9 = ivec2(gl_FragCoord.xy) + ivec2(-1, 0);
    int param_10 = 0;
    vec3 param_11 = tex2DFetch(samp_tex0, param_9, param_10).xyz;
    vec3 ML = toneMap(param_11);
    ivec2 param_12 = ivec2(gl_FragCoord.xy) + ivec2(1, 0);
    int param_13 = 0;
    vec3 param_14 = tex2DFetch(samp_tex0, param_12, param_13).xyz;
    vec3 MR = toneMap(param_14);
    ivec2 param_15 = ivec2(gl_FragCoord.xy) + ivec2(0, -1);
    int param_16 = 0;
    vec3 param_17 = tex2DFetch(samp_tex0, param_15, param_16).xyz;
    vec3 T = toneMap(param_17);
    ivec2 param_18 = ivec2(gl_FragCoord.xy) + ivec2(0, 1);
    int param_19 = 0;
    vec3 param_20 = tex2DFetch(samp_tex0, param_18, param_19).xyz;
    vec3 B = toneMap(param_20);
    float param_21 = T.x;
    float param_22 = M.x;
    float param_23 = B.x;
    float param_24 = fmax3(param_21, param_22, param_23);
    float param_25 = ML.x;
    float param_26 = MR.x;
    float param_27 = T.y;
    float param_28 = M.y;
    float param_29 = B.y;
    float param_30 = fmax3(param_27, param_28, param_29);
    float param_31 = ML.y;
    float param_32 = MR.y;
    float param_33 = T.z;
    float param_34 = M.z;
    float param_35 = B.z;
    float param_36 = fmax3(param_33, param_34, param_35);
    float param_37 = ML.z;
    float param_38 = MR.z;
    cMax = vec3(fmax3(param_24, param_25, param_26), fmax3(param_30, param_31, param_32), fmax3(param_36, param_37, param_38));
    float param_39 = T.x;
    float param_40 = M.x;
    float param_41 = B.x;
    float param_42 = fmin3(param_39, param_40, param_41);
    float param_43 = ML.x;
    float param_44 = MR.x;
    float param_45 = T.y;
    float param_46 = M.y;
    float param_47 = B.y;
    float param_48 = fmin3(param_45, param_46, param_47);
    float param_49 = ML.y;
    float param_50 = MR.y;
    float param_51 = T.z;
    float param_52 = M.z;
    float param_53 = B.z;
    float param_54 = fmin3(param_51, param_52, param_53);
    float param_55 = ML.z;
    float param_56 = MR.z;
    cMin = vec3(fmin3(param_42, param_43, param_44), fmin3(param_48, param_49, param_50), fmin3(param_54, param_55, param_56));
    curr_frame = ((((T * freqHigh_fragmentUniforms.filterweights0.y) + (ML * freqHigh_fragmentUniforms.filterweights0.w)) + (M * freqHigh_fragmentUniforms.filterweights1.x)) + (MR * freqHigh_fragmentUniforms.filterweights1.y)) + (B * freqHigh_fragmentUniforms.filterweights1.w);
    curr_frame_lowpass = ((((T * freqHigh_fragmentUniforms.filterweights3.y) + (ML * freqHigh_fragmentUniforms.filterweights3.w)) + (M * freqHigh_fragmentUniforms.filterweights4.x)) + (MR * freqHigh_fragmentUniforms.filterweights4.y)) + (B * freqHigh_fragmentUniforms.filterweights4.w);
    ivec2 param_57 = ivec2(gl_FragCoord.xy) + ivec2(-1);
    int param_58 = 0;
    vec3 param_59 = tex2DFetch(samp_tex0, param_57, param_58).xyz;
    vec3 TL = toneMap(param_59);
    ivec2 param_60 = ivec2(gl_FragCoord.xy) + ivec2(1, -1);
    int param_61 = 0;
    vec3 param_62 = tex2DFetch(samp_tex0, param_60, param_61).xyz;
    vec3 TR = toneMap(param_62);
    ivec2 param_63 = ivec2(gl_FragCoord.xy) + ivec2(-1, 1);
    int param_64 = 0;
    vec3 param_65 = tex2DFetch(samp_tex0, param_63, param_64).xyz;
    vec3 BL = toneMap(param_65);
    ivec2 param_66 = ivec2(gl_FragCoord.xy) + ivec2(1);
    int param_67 = 0;
    vec3 param_68 = tex2DFetch(samp_tex0, param_66, param_67).xyz;
    vec3 BR = toneMap(param_68);
    float param_69 = TL.x;
    float param_70 = TR.x;
    float param_71 = BL.x;
    float param_72 = fmax3(param_69, param_70, param_71);
    float param_73 = BR.x;
    float param_74 = cMax.x;
    float param_75 = TL.y;
    float param_76 = TR.y;
    float param_77 = BL.y;
    float param_78 = fmax3(param_75, param_76, param_77);
    float param_79 = BR.y;
    float param_80 = cMax.y;
    float param_81 = TL.z;
    float param_82 = TR.z;
    float param_83 = BL.z;
    float param_84 = fmax3(param_81, param_82, param_83);
    float param_85 = BR.z;
    float param_86 = cMax.z;
    cMax = (vec3(fmax3(param_72, param_73, param_74), fmax3(param_78, param_79, param_80), fmax3(param_84, param_85, param_86)) + cMax) * 0.5;
    float param_87 = TL.x;
    float param_88 = TR.x;
    float param_89 = BL.x;
    float param_90 = fmin3(param_87, param_88, param_89);
    float param_91 = BR.x;
    float param_92 = cMin.x;
    float param_93 = TL.y;
    float param_94 = TR.y;
    float param_95 = BL.y;
    float param_96 = fmin3(param_93, param_94, param_95);
    float param_97 = BR.y;
    float param_98 = cMin.y;
    float param_99 = TL.z;
    float param_100 = TR.z;
    float param_101 = BL.z;
    float param_102 = fmin3(param_99, param_100, param_101);
    float param_103 = BR.z;
    float param_104 = cMin.z;
    cMin = (vec3(fmin3(param_90, param_91, param_92), fmin3(param_96, param_97, param_98), fmin3(param_102, param_103, param_104)) + cMin) * 0.5;
    curr_frame += ((((TL * freqHigh_fragmentUniforms.filterweights0.x) + (TR * freqHigh_fragmentUniforms.filterweights0.z)) + (BL * freqHigh_fragmentUniforms.filterweights1.z)) + (BR * freqHigh_fragmentUniforms.filterweights2.x));
    curr_frame_lowpass += ((((TL * freqHigh_fragmentUniforms.filterweights3.x) + (TR * freqHigh_fragmentUniforms.filterweights3.z)) + (BL * freqHigh_fragmentUniforms.filterweights4.z)) + (BR * freqHigh_fragmentUniforms.filterweights5.x));
    vec3 param_105 = curr_frame;
    curr_frame = saturate(param_105);
    vec3 param_106 = curr_frame_lowpass;
    curr_frame_lowpass = saturate(param_106);
    cMax = max(curr_frame_lowpass, cMax);
    cMin = min(curr_frame_lowpass, cMin);
    vec2 param_107 = tc;
    vec3 _967 = computeVelocity(param_107);
    vec3 velocity = _967;
    vec3 prev_frame = vec3(0.0);
    vec2 tc_prev = tc * freqHigh_fragmentUniforms.prevtexsize.xy;
    vec2 uv = (tc_prev - velocity.xy) * freqHigh_fragmentUniforms.texsize.xy;
    vec2 iuv = floor(uv - vec2(0.5)) + vec2(0.5);
    vec2 fuv = uv - iuv;
    vec2 fuv2 = fuv * fuv;
    vec2 fuv3 = fuv * fuv2;
    vec2 w0 = fuv2 - ((fuv3 + fuv) * 0.5);
    vec2 w1 = ((fuv3 * 1.5) - (fuv2 * 2.5)) + vec2(1.0);
    vec2 w3 = (fuv3 - fuv2) * 0.5;
    vec2 w2 = ((vec2(1.0) - w0) - w1) - w3;
    vec2 wm = w1 + w2;
    vec2 fuvM = w2 / (w1 + w2);
    vec2 t0 = (iuv - vec2(1.0)) * freqHigh_fragmentUniforms.texsize.zw;
    vec2 t1 = (iuv + fuvM) * freqHigh_fragmentUniforms.texsize.zw;
    vec2 t2 = (iuv + vec2(2.0)) * freqHigh_fragmentUniforms.texsize.zw;
    vec4 param_108 = vec4(vec2(t0.x, t0.y), 0.0, 0.0);
    prev_frame = (tex2Dlod(samp_tex1, param_108).xyz * w0.x) * w0.y;
    vec4 param_109 = vec4(vec2(t1.x, t0.y), 0.0, 0.0);
    prev_frame += ((tex2Dlod(samp_tex1, param_109).xyz * wm.x) * w0.y);
    vec4 param_110 = vec4(vec2(t2.x, t0.y), 0.0, 0.0);
    prev_frame += ((tex2Dlod(samp_tex1, param_110).xyz * w3.x) * w0.y);
    vec4 param_111 = vec4(vec2(t0.x, t1.y), 0.0, 0.0);
    prev_frame += ((tex2Dlod(samp_tex1, param_111).xyz * w0.x) * wm.y);
    vec4 param_112 = vec4(vec2(t1.x, t1.y), 0.0, 0.0);
    prev_frame += ((tex2Dlod(samp_tex1, param_112).xyz * wm.x) * wm.y);
    vec4 param_113 = vec4(vec2(t2.x, t1.y), 0.0, 0.0);
    prev_frame += ((tex2Dlod(samp_tex1, param_113).xyz * w3.x) * wm.y);
    vec4 param_114 = vec4(vec2(t0.x, t2.y), 0.0, 0.0);
    prev_frame += ((tex2Dlod(samp_tex1, param_114).xyz * w0.x) * w3.y);
    vec4 param_115 = vec4(vec2(t1.x, t2.y), 0.0, 0.0);
    prev_frame += ((tex2Dlod(samp_tex1, param_115).xyz * wm.x) * w3.y);
    vec4 param_116 = vec4(vec2(t2.x, t2.y), 0.0, 0.0);
    prev_frame += ((tex2Dlod(samp_tex1, param_116).xyz * w3.x) * w3.y);
    vec3 param_117 = prev_frame;
    vec3 param_118 = toneMap(param_117);
    prev_frame = saturate(param_118);
    vec3 param_119 = (cMax - prev_frame) / (cMax - cMin);
    vec3 cubeBlendRatios = saturate(param_119);
    prev_frame = mix(cMax, cMin, cubeBlendRatios);
    vec3 param_120 = prev_frame;
    float luma_prev_frame = GetLuma(param_120);
    vec3 param_121 = cMax;
    float luma_max = GetLuma(param_121);
    vec3 param_122 = cMin;
    float luma_min = GetLuma(param_122);
    float curr_frame_edges = luma_max - luma_min;
    float prev_frame_edges = min(abs(luma_prev_frame - luma_max), abs(luma_prev_frame - luma_min));
    float param_123 = length(velocity.xy * freqHigh_fragmentUniforms.texsize.xy);
    float velocity_k = saturate(param_123);
    float k = (prev_frame_edges * (1.0 + (5.0 * velocity_k))) / 8.0;
    float param_124 = transparency_k + (k / (k + curr_frame_edges));
    float w = saturate(param_124);
    float param_125 = w;
    vec3 param_126 = mix(prev_frame, curr_frame, vec3(saturate(param_125)));
    vec3 _1324 = max(vec3(0.0), toneMapInverse(param_126));
    out_FragColor0 = vec4(_1324.x, _1324.y, _1324.z, out_FragColor0.w);
    vec4 param_127 = freqHigh_fragmentUniforms.motionblurparms;
    vec2 param_128 = velocity.xy;
    float param_129 = velocity.z;
    vec4 _1338 = OutputVelocityTarget(param_127, param_128, param_129);
    out_FragColor1 = vec4(_1338.zw.x, _1338.zw.y, out_FragColor1.z, out_FragColor1.w);
    out_FragColor1.x = 1.0 / out_FragColor1.x;
}

