#version 450
 #extension GL_EXT_multiview : enable
// Vk3DVision SinglePass
///////////////////////////////////////////////////////////////////////////////

#define STEREO 1

#define VIEW gl_ViewIndex

// Stereo and Mono Texture Sampling
#if(STEREO == 1)
	#define SAMPLER_2D sampler2DArray
	#define USAMPLER_2D usampler2DArray
	#define ISAMPLER_2D isampler2DArray
	#define IMAGE_2D image2DArray
	#define UIMAGE_2D uimage2DArray
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, vec3(uv, VIEW))
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, vec3(uv, VIEW), bias)
	#define IMAGE_LOAD(Image2D, pos) imageLoad(Image2D, ivec3(pos, VIEW))
	#define IMAGE_STORE(Image2D, pos, result) imageStore(Image2D, ivec3(pos, VIEW), result)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, ivec3(uv, VIEW), lod)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, vec3(uv, VIEW), lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, vec3(uv, VIEW), lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, vec3(uv, VIEW), comp)
	#define TEXTURE_GATHER1(Sampler, uv) textureGather(Sampler, vec3(uv, VIEW))
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, vec3(uv, VIEW), offset, comp)
	#define TEXTURE_GATHER_OFFSETS(Sampler, uv, offset) textureGatherOffsets(Sampler, vec3(uv, VIEW), offset)
	#define TEXTURE_GATHER_OFFSETS1(Sampler, uv, offset, comp) textureGatherOffsets(Sampler, vec3(uv, VIEW), offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod).xy
	#define TEXTURE_GRAD(Sampler, P, dPdx, dPdy) textureGrad(Sampler, vec3(P, VIEW), dPdx, dPdy)
#else
	#define SAMPLER_2D sampler2D
	#define USAMPLER_2D usampler2D
	#define ISAMPLER_2D isampler2D
	#define IMAGE_2D image2D
	#define UIMAGE_2D uimage2D
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, uv)
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, uv, bias)
	#define IMAGE_LOAD(Image2D, pos) imageLoad(Image2D, pos)
	#define IMAGE_STORE(Image2D, pos, result) imageStore(Image2D, pos, result)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, uv, lod)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, uv, lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, uv, lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, uv, comp)
	#define TEXTURE_GATHER1(Sampler, uv, comp) textureGather(Sampler, uv, comp)
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, uv, offset, comp)
	#define TEXTURE_GATHER_OFFSETS(Sampler, uv, offset) textureGatherOffsets(Sampler, uv, offset)
	#define TEXTURE_GATHER_OFFSETS1(Sampler, uv, offset, comp) textureGatherOffsets(Sampler, uv, offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod)
	#define TEXTURE_GRAD(Sampler, P, dPdx, dPdy) textureGrad(Sampler, P, dPdx, dPdy)
#endif 
 ///////////////////////////////////////////////////////////////////////////////


// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
struct Vk3DStereo
{
	vec4 stereo;
	vec4 custom_params;
};
layout(set = 0, binding = 30, std140) uniform Vk3DParams
{
	Vk3DStereo vk3d_params[2];
};

layout(set = 0, binding = 0, std140) uniform freqHigh_fragmentUniforms_ubo
{
    vec4 texsize;
    vec4 prevviewprojectionmatrixw;
    vec4 prevviewprojectionmatrixx;
    vec4 prevviewprojectionmatrixy;
    vec4 renderpositiontoviewtexture;
    vec4 motionblurparms;
} freqHigh_fragmentUniforms;

layout(set = 0, binding = 1, std140) uniform freqLow_fragmentUniforms_ubo
{
    vec4 projectionmatrixz;
    vec4 frustumvectl;
    vec4 frustumvectr;
    vec4 resolutionscale;
    vec4 frustumvecbl;
    vec4 frustumvecbr;
    vec4 globalvieworigin;
} freqLow_fragmentUniforms;

layout(set = 0, binding = 2) uniform SAMPLER_2D samp_tex4;
layout(set = 0, binding = 3) uniform SAMPLER_2D samp_tex2;
layout(set = 0, binding = 4) uniform SAMPLER_2D samp_tex0;

layout(location = 0) out vec4 out_FragColor0;
layout(location = 1) out vec4 out_FragColor1;

vec2 screenPosToTexcoord(vec2 pos, vec4 bias_scale)
{
    return (pos * bias_scale.zw) + bias_scale.xy;
}

vec4 tex2DFetch(SAMPLER_2D image, ivec2 texcoord, int mip)
{
    return TEXEL_FETCH(image, texcoord, mip);
}

float GetLinearDepth(float ndcZ, vec4 projectionMatrixZ, float rcpFarZ, bool bFirstPersonArmsRescale)
{
    float linearZ = projectionMatrixZ.w / (ndcZ + projectionMatrixZ.z);
    if (bFirstPersonArmsRescale)
    {
        float _141;
        if (linearZ < 1.0)
        {
            _141 = 10.0;
        }
        else
        {
            _141 = 1.0;
        }
        linearZ *= _141;
    }
    return linearZ * rcpFarZ;
}

vec4 tex2Dlod(SAMPLER_2D image, vec4 texcoord)
{
    return TEXTURE_LOD(image, texcoord.xy, texcoord.w);
}

vec3 computeVelocity(inout vec2 winPos)
{
    const ivec2 offsets[4] = {ivec2(0,0), ivec2(0,0), ivec2(0,0), ivec2(0,0)};
    vec4 depths = TEXTURE_GATHER_OFFSETS(samp_tex4, winPos, offsets);
    vec3 offsetMinZ = vec3(vec2(-1.0), depths.x);
    vec3 _181;
    if (depths.y < offsetMinZ.z)
    {
        _181 = vec3(vec2(1.0, -1.0), depths.y);
    }
    else
    {
        _181 = offsetMinZ;
    }
    offsetMinZ = _181;
    vec3 _199;
    if (depths.z < offsetMinZ.z)
    {
        _199 = vec3(vec2(-1.0, 1.0), depths.z);
    }
    else
    {
        _199 = offsetMinZ;
    }
    offsetMinZ = _199;
    vec3 _216;
    if (depths.w < offsetMinZ.z)
    {
        _216 = vec3(vec2(1.0), depths.w);
    }
    else
    {
        _216 = offsetMinZ;
    }
    offsetMinZ = _216;
    winPos += (offsetMinZ.xy * freqHigh_fragmentUniforms.texsize.zw);
    float param = offsetMinZ.z;
    vec4 param_1 = freqLow_fragmentUniforms.projectionmatrixz;
    float param_2 = 1.0;
    bool param_3 = true;
    float linearZ = GetLinearDepth(param, param_1, param_2, param_3);
    vec3 frustumVecX0 = mix(freqLow_fragmentUniforms.frustumvectl.xyz, freqLow_fragmentUniforms.frustumvectr.xyz, vec3(winPos.x * freqLow_fragmentUniforms.resolutionscale.z));
    vec3 frustumVecX1 = mix(freqLow_fragmentUniforms.frustumvecbl.xyz, freqLow_fragmentUniforms.frustumvecbr.xyz, vec3(winPos.x * freqLow_fragmentUniforms.resolutionscale.z));
    vec3 frustumVec = mix(frustumVecX1, frustumVecX0, vec3(1.0 - (winPos.y * freqLow_fragmentUniforms.resolutionscale.w)));
    vec3 world_pos = freqLow_fragmentUniforms.globalvieworigin.xyz + (frustumVec * linearZ);
    float rcpHW = 1.0 / ((world_pos.x * freqHigh_fragmentUniforms.prevviewprojectionmatrixw.x) + ((world_pos.y * freqHigh_fragmentUniforms.prevviewprojectionmatrixw.y) + ((world_pos.z * freqHigh_fragmentUniforms.prevviewprojectionmatrixw.z) + freqHigh_fragmentUniforms.prevviewprojectionmatrixw.w)));
    vec2 winPosPrev = vec2((world_pos.x * freqHigh_fragmentUniforms.prevviewprojectionmatrixx.x) + ((world_pos.y * freqHigh_fragmentUniforms.prevviewprojectionmatrixx.y) + ((world_pos.z * freqHigh_fragmentUniforms.prevviewprojectionmatrixx.z) + freqHigh_fragmentUniforms.prevviewprojectionmatrixx.w)), (world_pos.x * freqHigh_fragmentUniforms.prevviewprojectionmatrixy.x) + ((world_pos.y * freqHigh_fragmentUniforms.prevviewprojectionmatrixy.y) + ((world_pos.z * freqHigh_fragmentUniforms.prevviewprojectionmatrixy.z) + freqHigh_fragmentUniforms.prevviewprojectionmatrixy.w))) * rcpHW;
    vec4 param_4 = vec4(winPos, 0.0, 0.0);
    vec2 objV = tex2Dlod(samp_tex2, param_4).xy;
    vec2 _387;
    if (objV.x < 1.0)
    {
        _387 = objV;
    }
    else
    {
        _387 = winPos - (winPosPrev * freqLow_fragmentUniforms.resolutionscale.xy);
    }
    return vec3(_387, linearZ);
}

float saturate(float v)
{
    return clamp(v, 0.0, 1.0);
}

vec4 OutputVelocityTarget(vec4 mbParms, inout vec2 v, float z)
{
    vec2 v_orig = v;
    float alpha = 1.0;
    v *= (mbParms.z * alpha);
    float len = inversesqrt(dot(v, v));
    float param = mbParms.w * len;
    v *= saturate(param);
    v *= mbParms.xy;
    return vec4(v_orig, vec2(length(v), z));
}

void main()
{
    vec2 param = gl_FragCoord.xy;
    vec4 param_1 = freqHigh_fragmentUniforms.renderpositiontoviewtexture;
    vec2 tc = screenPosToTexcoord(param, param_1);
    vec3 curr_frame = vec3(0.0);
    vec3 curr_frame_lowpass = vec3(0.0);
    vec3 cMax = vec3(0.0);
    vec3 cMin = vec3(0.0);
    ivec2 param_2 = ivec2(gl_FragCoord.xy);
    int param_3 = 0;
    vec3 M = tex2DFetch(samp_tex0, param_2, param_3).xyz;
    vec2 param_4 = tc;
    vec3 _436 = computeVelocity(param_4);
    vec3 velocity = _436;
    vec3 _440 = max(vec3(0.0), M);
    out_FragColor0 = vec4(_440.x, _440.y, _440.z, out_FragColor0.w);
    vec4 param_5 = freqHigh_fragmentUniforms.motionblurparms;
    vec2 param_6 = velocity.xy;
    float param_7 = velocity.z;
    vec4 _453 = OutputVelocityTarget(param_5, param_6, param_7);
    out_FragColor1 = vec4(_453.zw.x, _453.zw.y, out_FragColor1.z, out_FragColor1.w);
    out_FragColor1.x = 1.0 / out_FragColor1.x;
}

